-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua = npcg.functua

local list = { }

functua.list = list

-------------------------------------------------------------------------------------
-- | Type Definitions | -------------------------------------------------------------
-------------------------------------------------------------------------------------

-- type List a :: { int, a }
-- A list is a sequence of items of which iteration is ordered (by key) and does not provide keys.

-- type Action a :: (a -> ())
-- An action is a function that takes "a" as an argument, and returns void.

-------------------------------------------------------------------------------------
-- | Iteration Functions | ----------------------------------------------------------
-------------------------------------------------------------------------------------

-- forEach :: Action a -> List a -> ()
-- Performs the same action on every element in a list.
function list.forEach(a, xs)
	for _,x in ipairs(xs) do
		a(x)
	end
end

-- map :: (a -> b) -> List a -> List b
-- Applies a function to every item in a table and returns the result.
-- Ordering is preserved.
function list.map(f, xs)
	local output = { }
	for i,v in ipairs(xs) do
		output[i] = f(v)
	end
	return output
end

-- filter :: (a -> bool) -> List a -> List a
-- Returns a list of all elements in a list that satisfy a predicate.
function list.filter(p, xs)
	local output = { }
	for _,v in ipairs(xs) do
		if p(v) then 
			table.insert(output, v)
		end
	end
	return output
end

-- foldr :: (a -> b -> b) -> b -> List a -> b
-- Applies an accumulator function on every element of a given list.
function list.foldr(op, val, xs)
	for _,v in ipairs(xs) do
		val = op(val, v)
	end
	return val
end

-------------------------------------------------------------------------------------
-- | Combination Functions | --------------------------------------------------------
-------------------------------------------------------------------------------------

-- insert :: List a -> a -> List a
-- Returns a list comprised of a given list and a given value.
-- Note: This is O(n) to preserve purity.
function list.insert(xs, x)
	local output = list.map(functua.id, xs)
	table.insert(output, x)
	return output
end


-- concat :: List (List a) -> List a
-- Concatenates a list of lists.
function list.concat(xs)
	local output = { }
	for _,v in ipairs(xs) do
		for _,vv in ipairs(v) do
			table.insert(output, vv)
		end
	end
	return output
end

-------------------------------------------------------------------------------------
-- | Cloning Functions | ------------------------------------------------------------
-------------------------------------------------------------------------------------

-- clone :: List a -> List a
-- Returns a shallow clone of a table.
function list.clone(xs)
	return list.map(functua.id, xs)
end

-------------------------------------------------------------------------------------
-- | Counting Functions | -----------------------------------------------------------
-------------------------------------------------------------------------------------

-- count :: { a } -> int
-- Counts the number of elements in a list.
function list.count(xs)
	return list.foldr(
    function(accum, x) return accum + 1 end, 0, 
    xs)
end

-- sum :: Num a => { a } -> a
-- Computes the sum of the items in the list.
-- Items must be numbers.
function list.sum(xs)
  return list.foldr(
    function(accum, x) return accum + x end, 0,
    xs)
end

-- intersect :: { a } -> { a } -> { a } 
-- Returns the intersection between two lists, aka the elements that are both in xs and ys.
-- intersect( {1},   {1,1} ) returns {1}
-- intersect( {1,1}, {1}   ) returns {1,1}
function list.intersect(xs, ys)
  -- Rough way to calculate an intersection between two lists.
  local set = { }
  local output = { }
  for _,y in ipairs(ys) do
    set[y] = true
  end
  for _,x in ipairs(xs) do
    if set[x] then
      table.insert(output, x)
    end
  end
  return output
end

-------------------------------------------------------------------------------------
-- | Query Functions | --------------------------------------------------------------
-------------------------------------------------------------------------------------

-- any :: (a -> bool) -> Table _ a -> bool
-- Returns whether any of the elements in a table satisfies a predicate.
function list.any(p, xs)
	for _,v in ipairs(xs) do
		if p(v) then
			return true
		end
	end
	return false
end

-- contains :: a -> { a } -> bool
-- Returns whether a value is contained in a list.
function list.contains(val, xs)
	return list.any(
		function(x) return x == val end, 
		xs)
end

-------------------------------------------------------------------------------------
-- | Skipping and Taking | ----------------------------------------------------------
-------------------------------------------------------------------------------------

-- skipWhile :: (a -> bool) -> { a } -> { a }
-- Returns a list where all items until the predicate is satisfied are ignored. The element where the predicate is true is included.
function list.skipWhile(p, xs)
  local output = { }
  local found = false
  for _,v in ipairs(xs) do
    if p(v) then
      found = true
    end
    if found then 
      table.insert(output, v)
    end
  end
  return output
end

-- takeWhile :: (a -> bool) -> { a } -> { a }
-- Returns a list where all items after the predicate is satisfied are ignored. The element where the predicate is true is not included.
function list.takeWhile(p, xs)
  local output = { }
  local found = false
  for _,v in ipairs(xs) do
    if   p(v) then return output
    else           table.insert(output, v) 
    end
  end
  return output
end

-- End of File.