-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua 	= npcg.functua

local stringLib = { }

functua.string = stringLib

-------------------------------------------------------------------------------------
-- | String |------------------------------------------------------------------------
-------------------------------------------------------------------------------------

-- searchF :: (string --> string) --> string --> [string] --> [string]
-- Searches for the existence of a string after mapping a function over a list of strings.
function stringLib.searchF(f, s, xs)
  return functua.list.filter(
    function(x) return string.match(f(x), s) end,
    xs)
end

-- combine :: string -> string
-- Concatenates two strings even if either is nil.
function stringLib.combine(x,y)
  return (x or "") .. (y or "")
end

-- combineWithSpace :: string -> string
-- Concatenates two strings with a whitespace in between. If either is nil or empty, that one will be ignores.
function stringLib.combineWithSpace(x,y)
  if      x == nil or x == "" then  return y
  elseif  y == nil or y == "" then  return x
  else                              return x .. " " .. y end
end

-- nilOrWhitespace :: string -> bool
-- Returns whether a string is nil or empty.
function stringLib.nilOrWhitespace(s)
  return s == nil or s == ""
end

-- nilOrWhitespace :: string -> string -> bool
-- Returns whether a string is nil, empty or equal to the other string.
function stringLib.nilOrWhitespaceOr(s, t)
  return stringLib.nilOrWhitespace(s) or (s == t)
end

-- ucFirst :: string -> string
-- Capitalizes the first letter of a string.
function stringLib.ucFirst(s)
  return s:gsub("^%l", string.upper)
end

-- lcFirst :: string -> string
-- Lower-cases the first letter of a string.
function stringLib.lcFirst(s)
  return s:gsub("^%u", string.lower)
end

-- upperCaseWordsHelper :: string -> string -> string
-- Capitalizes x, and concatenates with xs.
local function upperCaseWordsHelper(x, xs)
  return x:upper() .. xs:lower()
end

-- ucWords :: string -> string
-- Capitalizes every word in a string.
function stringLib.ucWords(s)
  return s:gsub("(%a)([%w_']*)", upperCaseWordsHelper)
end

-- underscoreToSpace :: string -> string
-- Replaces every occurance of "_" by a single whitespace.
function stringLib.underscoreToSpace(s)
  return s:gsub("_", " ")
end

-- doubleUnderscoreToApostrophe :: string -> string
-- Replaces every occurance of "__", two underscores, by "'", an apostrophe.
function stringLib.doubleUnderscoreToApostrophe(s)
	return s:gsub("__", "'")
end

-------------------------------------------------------------------------------------
-- | Fixing Words in strings |-------------------------------------------------------
-------------------------------------------------------------------------------------

-- fixGender :: Gender -> string -> string
-- Replaces occurances of xir-, xim- and xhe-macros by their gender-specific counterparts.
function stringLib.fixGender(gender, s)
  return
		stringLib.xir(gender, 
		stringLib.xim(gender,
		stringLib.xhe(gender, s)))
end

-- inverseFixGender :: string -> Gender
-- Replaces occurances of gender-specific words to their xir-, xim- or xhe-macro counterparts.
function stringLib.inverseFixGender(s)
  return
		stringLib.inverseXir(
		stringLib.inverseXim(
		stringLib.inverseXhe(s)))
end

-- xir :: Gender -> string -> string
-- Replaces occurances of xir-macros by their gender-specific counterpart.
function stringLib.xir(gender, s)
  if     gender ==        "male" then return s:gsub("%%xir",        "his") 
  elseif gender ==      "female" then return s:gsub("%%xir",        "her")
  elseif gender == "androgynous" then return s:gsub("%%xir", "his or her")
  elseif gender ==  "non-binary" then return s:gsub("%%xir",      "their")
  else                                return s:gsub("%%xhe",        "its")
	end
end

-- inverseXir :: string -> Gender
-- Replaces occurances of "his", "her" or "his or her" by their xir-macro counterpart.
function stringLib.inverseXir(s)
  local output = s
  output = output:gsub(       " his ", " %%xir ")
  output = output:gsub(       " her ", " %%xir ")
  output = output:gsub(" his or her ", " %%xir ")
  output = output:gsub(      " their", " %%xir ")
  return output
end

-- xim :: Gender -> string -> string
-- Replaces occurance of xim-macros by their gender-specific counterpart.
function stringLib.xim(gender, s)
  if     gender ==        "male" then return s:gsub("%%xim",        "him") 
  elseif gender ==      "female" then return s:gsub("%%xim",        "her")
  elseif gender == "androgynous" then return s:gsub("%%xim", "him or her")
  elseif gender ==  "non-binary" then return s:gsub("%%xim",       "them")
  else                                return s:gsub("%%xhe",         "it")
	end
end

-- inverseXim :: string -> Gender
-- Replaces occurances of "him", "her" or "him or her" by their xim-macro counterpart.
function stringLib.inverseXim(s)
  local output = s
  output = output:gsub(       " him ", " %%xim ")
  output = output:gsub(       " her ", " %%xim ")
  output = output:gsub(" him or her ", " %%xim ")
  output = output:gsub(      " them ", " %%xim ")
  return output
end

-- xhe :: Gender -> string -> string
-- Replaces occurance of xhe-macros by their gender-specific counterpart.
function stringLib.xhe(gender, s)
  if     gender ==        "male" then return s:gsub("%%xhe",        "he") 
  elseif gender ==      "female" then return s:gsub("%%xhe",       "she")
  elseif gender == "androgynous" then return s:gsub("%%xhe", "he or she")
  elseif gender ==  "non-binary" then return s:gsub("%%xhe",      "they")
  else                                return s:gsub("%%xhe",        "it")
	end
end

-- inverseXim :: string -> Gender
-- Replaces occurances of "he", "she" or "he or she" by their xhe-macro counterpart.
function stringLib.inverseXhe(s)
  local output = s
  output = output:gsub(        " he ", " %%xhe ")
  output = output:gsub(       " she ", " %%xhe ")
  output = output:gsub( " he or she ", " %%xhe ")
  output = output:gsub(      " they ", " %%xhe ")
  return output
end

-- isOrAre :: string --> string
-- Corrects instances where the form of "to be" does not fit the pronouns.
-- Example: " they is " is converted to " they are ".
function stringLib.isOrAre(s)
  local output = s
  output = output:gsub(       " he are ", " he is "       )
  output = output:gsub(      " she are ", " she is "      )
  output = output:gsub(" he or she are ", " he or she is ")
  output = output:gsub(      " they is ", " they are "    )
  return output
end

-- aOrAn :: string -> string
-- Prefixes a word with either "a" or "an" based on the word.
function stringLib.aOrAn(word)
  if string.find(string.sub(word, 1,1), "[AEIOUaeiou]") then return "an"
  else return "a" end
end

-- trySingular :: string --> string
-- Removes the last letter of a word to try and make it singular.
function stringLib.trySingular(word)
	return string.sub(word, 1, string.len(word) - 1)
end

-- eof