-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua 	= npcg.functua
local stringLib = npcg.functua.string
local util 			= npcg.functua.util

local io				= { }
local system    = { }

npcg.io					= io
npcg.system     = system
-- Checking traits to be valid will be postponed until a later version.

-----------------------------------------------------------------------
-- | Helper Functions | -----------------------------------------------
-----------------------------------------------------------------------

-- getLocation :: Zone, Subzone
-- Returns the zone and subzone that should be used for generation, taking settings into account.
local function getLocation()
	return
		-- Zone
		string.lower(util.eitherOr(
			-- If Zone is mocked
			npcg.options.savedVars["useMockZone"] and npcg.options.savedVars["useZones"],
			-- then
      npcg.options.savedVars["mockZone"],
			-- else
			GetZoneText())),
		-- Subzone
		string.lower(util.eitherOr(
			-- If Subzone is mocked
			npcg.options.savedVars["useMockSubZone"] and npcg.options.savedVars["useZones"] and npcg.options.savedVars["useSubZones"],
			-- then
			npcg.options.savedVars["mockSubZone"],
			-- else
			GetSubZoneText()))
end

-----------------------------------------------------------------------
---|            | -----------------------------------------------------
-- | Module: IO | -----------------------------------------------------
---|            | -----------------------------------------------------
-----------------------------------------------------------------------

-----------------------------------------------------------------------
-- | Writer Functions |------------------------------------------------
-----------------------------------------------------------------------

-- WriteToSavedVariables :: characterTemplate -> int -> void
-- Writes a characterTemplate to the SavedVariables in external storage, at a given saveSlot.
-- If the saveSlot already has an npc in it, it will be overriden.
function io.WriteToSavedVariables(npc, saveSlot)
	if not stringLib.nilOrWhitespace(npc.name) then
		savedNpcTemplates[saveSlot] = CharacterTemplate.clone(npc)
	else
		error("The npc need a name to be saved!")
	end
end

-- WriteToEmote :: characterTemplate -> void
-- Writes a characterTemplate to the EMOTE chat channel.
function io.WriteToEmote(npc)
	SendChatMessage(
		generateCharacterString(npc),
		"EMOTE")
end

-- WriteToChat :: characterTemplate -> void
-- Writes a link to characterTemplate to a chat channel.
function io.WriteToChat(npc)
	error("Not Yet Implemented Exception!")
end

-- WriteToFrame :: characterTemplate -> void
-- Writes a characterTemplate to the main frame.
function io.WriteToFrame(npc)
	local frame = npcGenerator_NPC_Frame

  frame.Traits.Age.EditBox:SetText(      stringLib.ucWords(                                    npc:getAge()        ))
  frame.Traits.Alignment.EditBox:SetText(stringLib.ucWords(                                    npc:getAlignment()  ))
  frame.Traits.Class.EditBox:SetText(    stringLib.ucWords(																		npc:getClass()			))
  frame.Traits.Faith.EditBox:SetText(    stringLib.ucWords(npcg.processSubstitutionString(npc,	npc:getFaith())			))
  frame.Traits.Feature.EditBox:SetText(  stringLib.ucFirst(npcg.processSubstitutionString(npc, npc:getFeature())		))
  frame.Traits.Gender.EditBox:SetText(   stringLib.ucWords(																		npc:getGender()			))
  frame.Traits.Name.EditBox:SetText(     stringLib.ucWords(npcg.processSubstitutionString(npc, npc:getName())			))
  frame.Traits.Race.EditBox:SetText(     stringLib.ucWords(																		npc:getRace()				))
  frame.Traits.Tribe.EditBox:SetText(    stringLib.ucWords(																		npc:getTribe()			))

  frame.Traits.Motivations.EditBox1:SetText(stringLib.ucFirst(npcg.processSubstitutionString(npc, npc:getMotivation(1))))
  frame.Traits.Motivations.EditBox2:SetText(stringLib.ucFirst(npcg.processSubstitutionString(npc, npc:getMotivation(2))))
  frame.Traits.Motivations.EditBox3:SetText(stringLib.ucFirst(npcg.processSubstitutionString(npc, npc:getMotivation(3))))
end

function io.WriteToCache(npc)
	error("Not Yet Implemented Exception!")
end

-----------------------------------------------------------------------
-- | Reader Functions |------------------------------------------------
-----------------------------------------------------------------------

function io.ReadFromSavedVariables(npc)
	-- Current open tab.
  local n = npcGenerator_NPC_Frame.Tabs.currentTab

  npcg.system.characterTemplates[n]:fromExisting(npc)
  io.WriteToFrame(npcg.system.getCurrentCharacterTemplate())
end

function io.ReadFromChat(npc)
	error("Not Yet Implemented Exception!")
end

function io.ReadFromFrame(npc)
	error("Not Yet Implemented Exception!")
end

function io.ReadFromCache(npc)
	error("Not Yet Implemented Exception!")
end

-----------------------------------------------------------------------
-- | Deleter Functions |-----------------------------------------------
-----------------------------------------------------------------------

-- DeleteFromSavedVariables :: int -> void
-- Removes a characterTemplate from the savedVariables in external storage.
function io.DeleteFromSavedVariables(index)
	table.remove(savedNpcTemplates, index)
end

-- DeleteFromFrame :: int -> void
-- Removes a characterTemplate from the user-interface.
function io.DeleteFromFrame(index)
	error("Not Yet Implemented Exception!")
end

-----------------------------------------------------------------------
-- | Writing to the User Interface | ----------------------------------
-----------------------------------------------------------------------

-- move to UI

local function onUIChange(characterTemplate)
  local frame = npcGenerator_NPC_Frame
  if not isFirstGlanceWellDefined(characterTemplate) then
    frame.BottomBar.PrintButton:Disable()
  else
    frame.BottomBar.PrintButton:Enable()
  end
  
  if not characterTemplate:isEmpty() then
    frame.BottomBar.ClearButton:Enable()
  else
    frame.BottomBar.ClearButton:Disable()
  end
  
  if not stringLib.nilOrWhitespace(characterTemplate.name) then
    frame.BottomBar.SaveButton:Enable()
  else
    frame.BottomBar.SaveButton:Disable()
  end
	
	-- Tribe
	local tribeContext = npcg.data.tribe.getTribeContext(npcg.system.getCurrentCharacterTemplate():getRace())
	local hasTribe = not stringLib.nilOrWhitespace(tribeContext)
	
	if not hasTribe then
		-- Disable()
		frame.Traits.Tribe.Label:SetTextColor(0.5, 0.5, 0.5)
		frame.Traits.Tribe.Label:SetText("Tribe")
		frame.Traits.Tribe.EditBox:SetTextColor(0.5, 0.5, 0.5)
		frame.Traits.Tribe.EditBox:SetText("")
		frame.Traits.Tribe.EditBox:Disable()
		frame.Traits.Tribe.RollButton:Disable()
	else
		-- Enable()
		frame.Traits.Tribe.EditBox:SetTextColor(1.0, 1.0, 1.0)
		frame.Traits.Tribe.EditBox:Enable()
		--frame.Tribe.EditBox:SetText("")
		frame.Traits.Tribe.Label:SetTextColor(1.0, 1.0, 1.0)
		frame.Traits.Tribe.Label:SetText(tribeContext)
		frame.Traits.Tribe.RollButton:Enable()
	end
end

-----------------------------------------------------------------------
---|                | -------------------------------------------------
-- | Module: System | -------------------------------------------------
---|                | -------------------------------------------------
-----------------------------------------------------------------------

-----------------------------------------------------------------------
-- | Reading from the User Interface | --------------------------------
-----------------------------------------------------------------------

-- readTrait :: { traitName, traitDefinition } -> traitContainerUI -> characterTemplate -> traitName -> int -> void
-- Reads the value of a trait from a traitContainer in the NPC Frame.
-- traitDefinitions :: T.B.D.
-- characterTemplate is the active table of data entries for a single NPC. Must comply with trait definitions.
-- index is used to retrieve a value from a trait consisting of multiple fields.
function npcg.system.readTrait(traitDefinitions, uiElement, characterTemplate, traitName, index)
	local trait = traitName
	
	-- CharacterTemplate:SetMotivation instead of CharacterTemplate:SetMotivations
	if traitDefinitions[traitName].hasMultipleFields then
		trait = stringLib.trySingular(traitName)
	end

	characterTemplate["set"..trait](
		characterTemplate,														-- self variable
		uiElement.Traits[traitName]:GetValue(index):lower(),	-- Consider processSubstitutionString or inverse.
		index)																				-- Used in traits with multiple fields
	
	-- Invalidate UI.
	onUIChange(characterTemplate)
end

-----------------------------------------------------------------------
-- | Rolling Traits | -------------------------------------------------
-----------------------------------------------------------------------

-- Getting Depricated.
function npcg.system.generateRandomTrait(traitName, characterTemplate, zone, subzone)
		local options = {
		Name 				= generateRandomName,
		Race 				= generateRandomRace,
		Tribe 			= generateRandomTribe,
		Gender 			= generateRandomGender,
		Age 				= generateRandomAge,
		Alignment 	= generateRandomAlignment,
		Faith 			= generateRandomFaith,
		Class 			= generateRandomClass,
		Feature 		= generateRandomFeature,
		Motivations	= generateRandomMotivation,
	}
	
	return options[traitName](characterTemplate, zone, subzone)
end

-- rollTrait :: { traitName, traitDefinition } -> traitContainerUI -> characterTemplate -> traitName -> int -> characterTemplate
-- Reads the value of a trait from a traitContainer in the NPC Frame.
-- traitDefinitions :: T.B.D.
-- characterTemplate is the active table of data entries for a single NPC. Must comply with trait definitions.
-- index is used to retrieve a value from a trait consisting of multiple fields.
function npcg.system.rollTrait(traitDefinitions, uiElement, characterTemplate, traitName, index)
	local trait = traitName
	
	-- CharacterTemplate:SetMotivation instead of CharacterTemplate:SetMotivations
	if traitDefinitions[traitName].hasMultipleFields then
		trait = stringLib.trySingular(traitName)
	end

	-- Generate
	characterTemplate["set"..trait](
		characterTemplate, -- self parameter
		npcg.system.generateRandomTrait(
			traitName,
			characterTemplate,
			getLocation()), -- getLocation is not CAPS proofed.
		index)
	-- Write
	npcg.io.WriteToFrame(characterTemplate) -- This should be a single trait
	-- Return
	return characterTemplate	-- Unsure why this returns
end

-----------------------------------------------------------------------
-- | Clearing Traits | ------------------------------------------------
-----------------------------------------------------------------------

-- clearTrait :: { traitName, traitDefinition } -> traitContainerUI -> characterTemplate -> traitName -> int -> characterTemplate
-- Reads the value of a trait from a traitContainer in the NPC Frame.
-- traitDefinitions :: T.B.D.
-- characterTemplate is the active table of data entries for a single NPC. Must comply with trait definitions.
-- index is used to retrieve a value from a trait consisting of multiple fields.
function npcg.system.clearTrait(traitDefinitions, uiElement, characterTemplate, traitName, index)
	local trait = traitName
	
	-- CharacterTemplate:SetMotivation instead of CharacterTemplate:SetMotivations
	if traitDefinitions[traitName].hasMultipleFields then
		trait = stringLib.trySingular(traitName)
		-- Maybe TraitTemplateExtraLong can automatically do this?
		if index == 1 then
			-- Shift up
			characterTemplate["set"..trait](characterTemplate, characterTemplate["get"..trait](characterTemplate, 2), 1)
			characterTemplate["set"..trait](characterTemplate, characterTemplate["get"..trait](characterTemplate, 3), 2)
		elseif index == 2 then
			-- Shift up
			characterTemplate["set"..trait](characterTemplate, characterTemplate["get"..trait](characterTemplate, 3), 2)		
		end
		characterTemplate["set"..trait](characterTemplate, "", 3)
	else -- single trait.
		characterTemplate["set"..trait](characterTemplate, "", index)
	end
	
	-- Write
	npcg.io.WriteToFrame(characterTemplate) -- Do this on a trait per trait basis!
end

-- End of File.