-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua   = npcg.functua
local stringLib = npcg.functua.string
local util      = npcg.functua.util

-------------------------------------------------------------------------------------
-- | Variable Definitions |----------------------------------------------------------
-------------------------------------------------------------------------------------

-- Active options, these represent the live, non-cache states of the addon.
-- These should be read from the IgnoreEnemyEmotesDB on loading, and flushed to it on unloading.
-- These should be read from the cache on confirming changes.
local savedVars = { }

-- Cached options.
-- These should be flushed to the savedVars on confirming changes.
-- These should be cleared when leaving the options menu.
local cache     = { }

-- Default options. These cannot be changed after being set.
-- These must be explicitly defined for every savedVar.
local defaults  = {
  ["displayMinimapIcon"]  =  true,
  ["lockToMinimap"]       = false,
  ["enableTooltips"]      =  true,
  ["maximizeOnGenerate"]  =  true,
}

-------------------------------------------------------------------------------------
-- | Helper Functions |--------------------------------------------------------------
-------------------------------------------------------------------------------------

-- Set up a Checkbox, setting it's description, links it to a savedVar, set up the cache var, handles color changes based on enabling/disabling and sets up the tooltips.
local function setUpCheckboxHelper(checkbox, savedVarName, descriptionText, tooltipWhenChecked, tooltipWhenNotChecked)
  -- Set the name for the description.
  getglobal(checkbox:GetName() .. "Text"):SetText(descriptionText)
  -- On Show retrieve checked status from savedVars.
  checkbox:SetScript(   "OnShow", function(self) self:SetChecked(savedVars[savedVarName]) end)
  -- On Hide clear cache.
  checkbox:SetScript(   "OnHide", function(self) cache[savedVarName] = nil end)
  -- Post Click set cache, not value!
  checkbox:SetScript("PostClick", function(self) cache[savedVarName] = self:GetChecked() print("npcG Output: cache[\"" .. savedVarName .. "\"] = \"" .. tostring(cache[savedVarName]) .. "\"") end)
  checkbox:SetScript(  "OnClick", function(self) end) -- Defined because Blizz code will throw an error otherwise.
  -- Tooltip
  checkbox:SetScript(  "OnEnter", function(self) SetHelperTextOption(self:GetChecked() and tooltipWhenNotChecked or tooltipWhenChecked, self) end)
  -- Fade text based on checked status.
  checkbox:SetScript( "OnEnable", function(self) getglobal(self:GetName() .. "Text"):SetTextColor(1.0, 1.0, 1.0) end)
  checkbox:SetScript("OnDisable", function(self) getglobal(self:GetName() .. "Text"):SetTextColor(0.5, 0.5, 0.5) end)
end


-- Set up an Editbox, linking it to a savedVar, setting it's default text, providing a function to detect invalid input and corrects it, and sets up tooltips.
local function setUpEditboxHelper(editbox, savedVarName, defaultText, isValidText, tooltipText)
  
  -- On Show retrieve checked status from savedVars.
  editbox:SetScript(   "OnShow", function(self)
    self:SetText(isValidText(savedVars[savedVarName]) and savedVars[savedVarName] or defaultText) end)
  -- On Hide clear cache.
  editbox:SetScript(   "OnHide", function(self) cache[savedVarName] = nil end)

  -- Tooltip
  editbox:SetScript(  "OnEnter", function(self) SetHelperTextOption(tooltipText, self) end)
  -- Fade text based on checked status.
  editbox:SetScript( "OnEnable", function(self) self:SetTextColor(0.75, 0.75, 0.75) end)
  editbox:SetScript("OnDisable", function(self) self:SetTextColor(0.50, 0.50, 0.50) end)
  
  -- Parsing the input
  editbox:SetScript("OnEditFocusGained", function(self)
    self:SetTextColor(1.00, 1.00, 1.00)
    if self:GetText() == defaultText then self:SetText("") end
  end)
  editbox:SetScript("OnEditFocusLost", function(self)
    if not isValidText(self:GetText()) then self:SetText(defaultText) end
    self:SetTextColor(0.75, 0.75, 0.75)
    cache[savedVarName] = self:GetText() end)
end

-- Makes the enabled state of a frame depend on a given checkbox.
local function dependencyHelper(frame, dependency)
  
  -- Refresh function
  local refresh = function(dep)
    if    dep:IsEnabled() and dep:GetChecked() 
    then  frame:Enable()
    else  frame:Disable() end
  end
  
  dependency:HookScript("PostClick", function() refresh(dependency) end)
  dependency:HookScript( "OnEnable", function() refresh(dependency) end)
  dependency:HookScript("OnDisable", function() refresh(dependency) end)
  
  frame:HookScript("OnShow", function() refresh(dependency) end)
end


-------------------------------------------------------------------------------------
-- | Event Hooks and Handlers |------------------------------------------------------
-------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------
---- | Loading and Unloading |-------------------------------------------------------
-------------------------------------------------------------------------------------

-- Loads all options from external storage to npcg.options.savedVars.
-- To be called on ADDON_LOADED.
function npcg.loadInterfacePanelOptions(optionsPanelParent)
  -- Set the shorthand for npcg.options.savedVars.
  savedVars = npcg.options.savedVars

  -- Load savedVars used in this panel.
  functua.table.forEachKey(
    function(k) savedVars[k] = savedVars[k] or defaults[k] end,
    defaults)

  -- Set up the panel.
  local panel   = npcGenerator_Options_UI_Panel
	panel.name 		= "Interface"
	panel.parent 	= optionsPanelParent.name
  
  -- Set up okay button handler.
	panel.okay 		= function()
      -- Reset Minimap Icon if newly locked ot the minimap.
    if cache["lockToMinimap"] and not savedVars["lockToMinimap"] then npcg.minimapIcon:Reset() end  -- npcGenerator_minimapButton_Reset is local!!
  
    -- Flush cache to savedVars.
    functua.table.forEachKey(
      function(k)
        savedVars[k]  = cache[k]
        print("npcG Output: savedVars[\"" .. k .. "\"] = \"" .. tostring(savedVars[k]) .. "\"")
        cache[k]      = nil end,
      cache)
    
    -- Maybe.. fire an event instead of directly calling npcg.system.
    npcg.system.remakeGenerator() -- Defined in npcGenerator.lua. Name subject to change to npcg.system.remakeGenerators()
	end
  
  -- Set up default button handler.
	panel.default = function()
    functua.table.forEachKey(
      function(k) savedVars[k] = defaults[k] end,
      defaults)
	end
  
  -- Set up the cancel handler. This will clear the cache.
  panel.cancel  = function()
    functua.table.forEachKey(
      function(k)
        cache[k] = nil end,
      cache)
  end

  -- Set up the checkboxes, argument 1 is the checkbox, argument 2 is their related savedVar.
  setUpCheckboxHelper(panel.CheckMinimap,                   "displayMinimapIcon", "Minimap Icon", "Toggle the Minimap Icon.", "Toggle the Minimap Icon.")
  setUpCheckboxHelper(panel.LockMinimap,                    "lockToMinimap",      "Lock to Minimap", "Check to lock the Minimap Icon to the Minimap.", "Uncheck the Minimap Iconn to the Minimap.")
  setUpCheckboxHelper(panel.CheckTooltipsGeneral,           "enableTooltips",     "Enable Tooltips", "Check to enable tooltips", "Uncheck to disable tooltips")
  setUpCheckboxHelper(panel.CheckMaximizeWindowOnGenerate,  "maximizeOnGenerate", "Maximize on pressing Generate", "Check to maximize the NpcGenerator window when pressing \"Generate\"", "Uncheck to not maximize the NpcGenerator window when pressing \"Generate\"")
  
  -- Disables a child element when their parent is unchecked or disabled.
  dependencyHelper(panel.LockMinimap, panel.CheckMinimap)
  
	InterfaceOptions_AddCategory(npcGenerator_Options_UI_Panel)
end

-- eof