-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua   = npcg.functua
local stringLib = npcg.functua.string
local util      = npcg.functua.util

-------------------------------------------------------------------------------------
-- | Helper Functions |--------------------------------------------------------------
-------------------------------------------------------------------------------------

-- Set up a Checkbox, setting it's description, links it to a savedVar, set up the cache var, handles color changes based on enabling/disabling and sets up the tooltips.
local function setUpCheckboxHelper(checkbox, savedVarName, descriptionText, tooltipWhenChecked, tooltipWhenNotChecked)
  -- Set the name for the description.
  getglobal(checkbox:GetName() .. "Text"):SetText(descriptionText)
  -- On Show retrieve checked status from savedVars.
  checkbox:SetScript(   "OnShow", function(self) self:SetChecked(npcg.options.savedVars[savedVarName]) end)
  -- Post Click set savedVar, only directly set savedVar through input without confirmation in the Quick Menu!
  checkbox:SetScript("PostClick", function(self) npcg.options.savedVars[savedVarName] = self:GetChecked() npcg.system.remakeGenerator() end)
  checkbox:SetScript(  "OnClick", function(self) end) -- Defined because Blizz code will throw an error otherwise.
  -- Tooltip
  checkbox:SetScript(  "OnEnter", function(self) SetHelperTextOption(self:GetChecked() and tooltipWhenNotChecked or tooltipWhenChecked, self) end)
  -- Fade text based on checked status.
  checkbox:SetScript( "OnEnable", function(self) getglobal(self:GetName() .. "Text"):SetTextColor(1.0, 1.0, 1.0) end)
  checkbox:SetScript("OnDisable", function(self) getglobal(self:GetName() .. "Text"):SetTextColor(0.5, 0.5, 0.5) end)
end


-- Set up an Editbox, linking it to a savedVar, setting it's default text, providing a function to detect invalid input and corrects it, and sets up tooltips.
local function setUpEditboxHelper(editbox, savedVarName, defaultText, isValidText, tooltipText)
  
  -- On Show retrieve checked status from savedVars.
  editbox:SetScript(   "OnShow", function(self)
    self:SetText(isValidText(npcg.options.savedVars[savedVarName]) and npcg.options.savedVars[savedVarName] or defaultText) end)

  -- Tooltip
  editbox:SetScript(  "OnEnter", function(self) SetHelperTextOption(tooltipText, self) end)
  -- Fade text based on checked status.
  editbox:SetScript( "OnEnable", function(self) self:SetTextColor(0.75, 0.75, 0.75) end)
  editbox:SetScript("OnDisable", function(self) self:SetTextColor(0.50, 0.50, 0.50) end)
  
  -- Parsing the input
  editbox:SetScript("OnEditFocusGained", function(self)
    self:SetTextColor(1.00, 1.00, 1.00)
    if self:GetText() == defaultText then self:SetText("") end
  end)
  editbox:SetScript("OnEditFocusLost", function(self)
    if not isValidText(self:GetText()) then self:SetText(defaultText) end
    self:SetTextColor(0.75, 0.75, 0.75)
    npcg.options.savedVars[savedVarName] = self:GetText()
    npcg.system.remakeGenerator() end)
end

-- Makes the enabled state of a frame depend on a given checkbox.
local function dependencyHelper(frame, dependency)
  
  -- Refresh function
  local refresh = function(dep)
    if    dep:IsEnabled() and dep:GetChecked() 
    then  frame:Enable()
    else  frame:Disable() end
  end
  
  dependency:HookScript("PostClick", function() refresh(dependency) end)
  dependency:HookScript( "OnEnable", function() refresh(dependency) end)
  dependency:HookScript("OnDisable", function() refresh(dependency) end)
  
  frame:HookScript("OnShow", function() refresh(dependency) end)
end



function npcGenerator_settings_Frame_OnLoad()
  local frame = npcGenerator_settings_Frame
  tinsert(UISpecialFrames, frame:GetName())
  
  
  -- Make sure no more than one setting screen is open at the same time. Might be used later.
  npcGenerator_NPC_Frame.ExpandRightButton:SetScript("OnClick", function()
    if frame:IsVisible() then
      frame:Hide()
    else
      if frame:GetParent().activeExpandedFrame then
        frame:GetParent().activeExpandedFrame:Hide()
      end
      frame:GetParent().activeExpandedFrame = frame
      frame:Show()
    end
  end)
  
  npcGenerator_NPC_Frame.ExpandRightButton:SetScript("OnEnter", function()
    SetHelperText("Customize the generator", npcGenerator_NPC_Frame.ExpandRightButton)
  end)
end

--[[
function npcGenerator_settings_Frame_EditBox_MockZone_OnShow(self)
	self:SetText(util.eitherOr(
		not stringLib.nilOrWhitespace(npcg.options.savedVars["mockZone"]),
		npcg.options.savedVars["mockZone"],
		self.defaultText))
	self.Refresh()
end
]]

--[[
function npcGenerator_settings_Frame_EditBox_MockSubZone_OnShow(self)
	self:SetText(util.eitherOr(
		not stringLib.nilOrWhitespace(npcg.options.savedVars["mockSubZone"]),
		npcg.options.savedVars["mockSubZone"],
		self.defaultText))
	self.Refresh()
end
]]


function npcg.loadQuickPanelOptions()
    
  -- Set Title text
  npcGenerator_settings_Frame.Title.Label:SetText("Quick Menu")
  
  -- Set up checkboxes. Note: These edit savedVars immediately!
  local panel = npcGenerator_settings_Frame.Panel
  setUpCheckboxHelper(panel.ForceFactionToggle, "strictFactionChecking",  "Ban Opposing Faction", "Check to allow only your faction to appear", "Uncheck to allow all factions to appear")
  setUpCheckboxHelper(panel.ContextToggle,      "contextChecking",        "Unless Appropriate", "Check to make exceptions based on location or other factors", "Uncheck to not make exceptions based on location or other factors")
  setUpCheckboxHelper(panel.ZoneToggle,         "useZones",               "Use Zones", "Check to involve the location in the generation", "Uncheck to not involve the location in the generation")
  setUpCheckboxHelper(panel.MockZoneToggle,     "useMockZone",            "Fake Zone to be:", "Check to use a location specified instead of the in-game location", "Uncheck to use the in-game location and not one specified")
  setUpCheckboxHelper(panel.SubZoneToggle,      "useSubZones",            "Use Subzones", "Check to involve the subzone in the generation", "Uncheck to not involve the subzone in the generation")
  setUpCheckboxHelper(panel.MockSubZoneToggle,  "useMockSubZone",         "Fake Subzone to be", "Check to use a sub location specified instead of the in-game location", "Uncheck to use the in-game location and not one specified")
  setUpCheckboxHelper(panel.FriendsButton,      "haveFriendsAppear",      "Allow saved npcs to appear", "Check to allow npcs that are saved to occasionally pop up", "Uncheck to only use newly generated npcs")
  
  -- Set up editboxes. Note: These edit savedVars immediately!
  setUpEditboxHelper(panel.MockZoneEditBox,     "mockZone",               "Enter location here.", function(s) return not stringLib.nilOrWhitespace(s) end, "Enter a location to use instead of the in-game location.")
  setUpEditboxHelper(panel.MockSubZoneEditBox,  "mockSubZone",            "Enter location here.", function(s) return not stringLib.nilOrWhitespace(s) end, "Enter a location to use instead of the in-game location.")
  
  dependencyHelper(panel.ContextToggle,       panel.ForceFactionToggle)
  dependencyHelper(panel.MockZoneToggle,      panel.ZoneToggle)
  dependencyHelper(panel.MockZoneEditBox,     panel.MockZoneToggle)
  dependencyHelper(panel.SubZoneToggle,       panel.ZoneToggle)
  dependencyHelper(panel.MockSubZoneToggle,   panel.SubZoneToggle)
  dependencyHelper(panel.MockSubZoneEditBox,  panel.MockSubZoneToggle)
  
  -- Tooltips.
  npcGenerator_settings_Frame.CloseButton:SetScript("OnEnter", function(self) SetHelperText("Close the quick settings window", self) end)
  
end

-- eof